/*!
  \file
  \brief C[Ǘ

  \author Satofumi KAMIMURA

  $Id$

  \todo LogManager ܂肪 SdlSurface ƓȂ΂
  \todo remove ƂɁAĕ`悷邱Ƃۏ؂
*/

#include "LayerManager.h"
#include "Layer.h"
#include "SurfaceInterface.h"
#include "ComponentInterface.h"
#include "GuiColors.h"
#include "EvaluateArea.h"
#include "SdlUtils.h"
#include "LogManager.h"
#include <string>

using namespace beego;


struct LayerManager::pImpl {
  typedef std::vector<Layer*> LayerList;
  SurfaceList& surface_list;
  LayerList layer_list;
  SDL_Surface* scr;
  Uint32 back_color;
  size_t width;
  size_t height;
  std::vector<SDL_Rect> back_rects;

  pImpl(SurfaceList& surfaces)
    : surface_list(surfaces), scr(SDL_GetVideoSurface()), width(0), height(0) {
    if (scr == NULL) {
      LogManager* log = LogManager::getObject();
      std::string message =
	"LayerManager::pImpl(): Screen surface must be created.";
      log->write(LogManager::Warning, message.c_str());
      return;
    }
    width = scr->w;
    height = scr->h;
    back_color = getSdlColor(scr, Black);
  }
};


LayerManager::LayerManager(SurfaceList& surface_list)
  : pimpl(new pImpl(surface_list)) {
}


LayerManager::~LayerManager(void) {
}


void LayerManager::applyInput(ComponentList& components,
			      const InputEvent& event) {

  for (ComponentList::iterator it = components.begin();
       it != components.end(); ++it) {
    (*it)->applyInput(event);
  }
}


void LayerManager::redraw(std::vector<SDL_Rect>& update_rects,
			  SurfaceList& surfaces, size_t ticks) {

  if (! pimpl->scr) {
    return;
  }

  // wiFł̓hԂ
  for (std::vector<SDL_Rect>::iterator it = pimpl->back_rects.begin();
       it != pimpl->back_rects.end(); ++it) {
    SDL_FillRect(pimpl->scr, &*it, pimpl->back_color);
    update_rects.push_back(*it);
  }
  pimpl->back_rects.clear();

  for (SurfaceList::reverse_iterator rit = surfaces.rbegin();
       rit != surfaces.rend(); ++rit) {
    bool is_changed = (*rit)->surface->isChanged() || (*rit)->position_changed;

    // !!! ύXƈړ͈Ⴄ̂A
    // !!! -> 񓧉߂̏ꍇAύX̏ꍇ͏㏑邾ł
    // !!! -> ߂̏ꍇAύX̏ꍇł폜Kv
    // !!!   -> ӖŁA߂̏ꍇ́A폜ĕ`悷̂Ɠ
    // !!!   -> tɌƁA񓧉߂̏ꍇ

    // !!! ړ
    // !!! ̈ʒu𑼂̃T[tFXōĕ`悵ĂAVʒuɕ`s
    // !!! ƂẮA폜Ăǉ̂Ɠ

    // !!! 폜
    // !!! ̈ʒu𑼂̃T[tFXōĕ`悷

    // ǉ
    // !!! T[tFX̕`s
    // !!! XgOɌĕ]ƂAÕT[tFXŏdȂȂ΁Aĕ`
    // !!! T[tFẌ̗ŁAOɂ镔݂̂ĕ`悷
    // !!! OɂeT[tFXɑ΂āÄ̕]s
    // !!! -> 񓧉߃T[tFX́A`̈SɎ擾

    // !!! 폜 := ̃T[tFXŗ̈ĕ`悷
    // !!! ǉ := \ʂɂ镔ɑ΂ĕ`s
    // !!! ړ := 폜 -> ǉ
    // !!! ύX := 폜 -> ǉ

    // !!! 邩ǂ̕]́AOsׂ
    // !!! 폜Aǉ̗̈ɑ΂āAOĕ`悷ǂ̃T[tFX]
    // !!! -> 폜AǉȀԂ͂ǂׂȂ̂ȁH
    // !!! -> 폜́A폜\bhĂяoꂽƂɊKv
    // !!!    -> ܂AȂƂłȂ邯ǂȁ`

    // !!! 폜́A폜T[tFX̕\ʂǂȂĂ邩]
    // !!! -> \ʂ̉ӏ̂݁Aĕ]΂悢

    // !!! ǉ́A ǉԂŕ\ʂǂȂ邩A]΂悢
    // !!! -> ǉT[tFXOɌĕ]΂悳
    // !!! -> ǉ́A폜ƈႢΏۃT[tFXcĂ̂ŕ]͈ꊇōs

    if (is_changed) {
      // 폜
      // !!!

      // ǉ
      // !!!

      // !!! Au
      (*rit)->surface->draw(update_rects, &(*rit)->position,
			    &(*rit)->area, ticks);
      continue;
    }
  }
}


void LayerManager::push_front(Layer* layer) {
  pimpl->layer_list.insert(pimpl->layer_list.begin(), layer);
  layer->registerManager(this);

  // !!! ̂ƂɁASurfaceList ̍XVs
}


void LayerManager::push_back(Layer* layer) {

  pimpl->layer_list.push_back(layer);
  layer->registerManager(this);
}


void LayerManager::insert(const Layer* dst, Layer* layer) {

  pImpl::LayerList::iterator it = find(pimpl->layer_list.begin(),
				       pimpl->layer_list.end(), dst);
  pimpl->layer_list.insert(it, layer);
  layer->registerManager(this);
}


void LayerManager::remove(Layer* layer) {

  pImpl::LayerList::iterator it = find(pimpl->layer_list.begin(),
				       pimpl->layer_list.end(), layer);
  if (it != pimpl->layer_list.end()) {
    pimpl->layer_list.erase(it);
  }
}


void LayerManager::createComponentList(ComponentList& components) {

  // !!! NA[UɔCȂĂ̂ȁH
  components.clear();

  for (pImpl::LayerList::iterator it = pimpl->layer_list.begin();
       it != pimpl->layer_list.end(); ++it) {
    (*it)->addComponentList(components);
  }
}


void LayerManager::createSurfaceList(SurfaceList& surfaces,
				     const ComponentList& components) {

  // 폜̈̍XV

  // !!! ɁÂ͐p̊֐ɂĂ܂܂傤

  // !!! Ƃ肠폜͈̔͂ňx`悵Ă܂
  // !!! {́Aĕ`͍Ōɂ܂Ƃ߂čs΂悢͂
  std::vector<SDL_Rect> update_rects;
  for (SurfaceList::iterator it = surfaces.begin();
       it != surfaces.end(); ++it) {
    std::vector<SDL_Rect>& redraw_rects = (*it)->redraw_rects;
    for (std::vector<SDL_Rect>::iterator draw_it = redraw_rects.begin();
	 draw_it != redraw_rects.end(); ++draw_it) {
      SDL_Rect draw_area;
      set_SdlRect(&draw_area, draw_it->x - (*it)->position.x,
		  draw_it->y - (*it)->position.y, draw_it->w, draw_it->h);
      (*it)->surface->draw(update_rects, &*draw_it, &draw_area, 0);
      //fprintf(stderr, "(%d, %d, %d, %d), ", draw_area.x, draw_area.y, draw_area.w, draw_area.h);
    }
    redraw_rects.clear();
  }

  // !!! `AƂ肠Aupdate_rects Ă܂
  // !!! 蒼ׂ
  // !!! `揈AC[̒łĂ̂
  // !!! ꏊAɈڂA̕`揈ɎĂ
  if (pimpl->scr) {
    SDL_UpdateRects(pimpl->scr, update_rects.size(), &update_rects[0]);
  }
  // !!! ܂łAsurfaces NAǑォÂ

  // !!! NA[UɔCȂĂ̂ȁH
  surfaces.clear();

  SDL_Rect area;
  set_SdlRect(&area, 0, 0, pimpl->width -1, pimpl->height -1);

  for (ComponentList::const_iterator it = components.begin();
       it != components.end(); ++it) {
    (*it)->addSurfaceList(surfaces, &area);
  }

#if 0
  for (SurfaceList::iterator it = surfaces.begin();
       it != surfaces.end(); ++it) {
    fprintf(stderr, "%p, ", &it->surface);
  }
  fprintf(stderr, "\n");
#endif
}


void LayerManager::removeSurface(SurfaceList& remove_list) {
  //const SDL_Rect* remove_rect) {
  //void LayerManager::removeSurface(DrawSurface& draw_surface,
  //				 const SDL_Rect* remove_rect) {

  // O폜T[tFX܂ŁAĕ`̈]
  //SurfaceList::iterator own_it = pimpl->surface_list.end();

  // !!! Ƃ肠ASʂɑ΂ĕ]s
  // !!! ʕ́A{͂Ǝ擾ׂ
  SDL_Rect all_area;
  set_SdlRect(&all_area, 0, 0, 640, 480);
  std::vector<SDL_Rect> redraw_areas;
  redraw_areas.push_back(all_area);

  // ̈
  for (std::vector<SDL_Rect>::iterator redraw_it = redraw_areas.begin();
       redraw_it != redraw_areas.end(); ++redraw_it) {

    // 폜T[tFX
    // !!! ŁA폜T[tFXԂŁAdȂ̈]Ă悳
    for (SurfaceList::iterator remove_it = remove_list.begin();
	 remove_it != remove_list.end(); ++remove_it) {

      std::vector<SDL_Rect> remove_area;
      add_SdlRect(remove_area,
		  (*remove_it)->position.x, (*remove_it)->position.y,
		  (*remove_it)->area.w, (*remove_it)->area.h);
      //fprintf(stderr, "remove_area: %d, %d, %d, %d\n", (*remove_it)->position.x, (*remove_it)->position.y, (*remove_it)->area.w, (*remove_it)->area.h);

      std::vector<SDL_Rect> common_area; // gȂ
      std::vector<SDL_Rect> unique_area; // 폜T[tFX̌Ă镔
      bool front_surface = true;

      // ǗĂT[tFX
      for (SurfaceList::iterator surface_it = pimpl->surface_list.begin();
	   surface_it != pimpl->surface_list.end(); ++surface_it) {

	SDL_Rect surface_area;
	set_SdlRect(&surface_area,
		    (*surface_it)->position.x, (*surface_it)->position.y,
		    (*surface_it)->area.w, (*surface_it)->area.h);

	if (front_surface) {
	  // OʃT[tFXƂ̕]
	  // Ă̈ remove_area ɓo^Ă

	  if (*surface_it == *remove_it) {
	    //if (surface_it == remove_it) {
	    //fprintf(stderr, "id: %d, %d\n", surface_it->id, remove_it->id);
	    //if (surface_it->id == remove_it->id) {
	    // T[tFX܂ŕ]A͔wʂ̕]s
	    front_surface = false;

#if 1
	    //fprintf(stderr, "redraw_rects: %d\n", (*surface_it)->redraw_rects.size());
	    // T[tFX͍폜邽߁Ao^Ăĕ`͎̈ɓn
	    // !!! ̏AӖ̂ȁH
	    for (std::vector<SDL_Rect>::iterator it =
		   (*surface_it)->redraw_rects.begin();
		 it != (*surface_it)->redraw_rects.end(); ++it) {
	      remove_area.push_back(*it);
	    }
#endif

	    // T[tFX̓o^폜
	    surface_it = pimpl->surface_list.erase(surface_it);
	    --surface_it;

	    //fprintf(stderr, "mathc own surface\n");
	    continue;
	  }

	  // 폜T[tFX̌Ă镔 remove_area ɓo^
	  unique_area.clear();
	  common_area.clear();
	  for (std::vector<SDL_Rect>::iterator unique_it = remove_area.begin();
	       unique_it != remove_area.end(); ++unique_it) {
	    evaluateArea(common_area, unique_area, *unique_it, surface_area);
	  }
	  remove_area = unique_area;

	} else {
	  // wʃT[tFXƂ̕]
	  // remove_area ̗̈AwʂɂT[tFXɍĕ`悳

	  unique_area.clear();
	  for (std::vector<SDL_Rect>::iterator it = remove_area.begin();
	       it != remove_area.end(); ++it) {

	    // dȂ蔻sAcommon ̃T[tFXAunique ̃T[tFX
	    evaluateArea((*surface_it)->redraw_rects,
			 unique_area, *it, surface_area);
	    //fprintf(stderr, "evaluate redraw_rects: %d\n", (*surface_it)->redraw_rects.size());
#if 0
	    fprintf(stderr, "%d, %d, %d, %d\n", it->x, it->y, it->w, it->h);
	    fprintf(stderr, "%d, %d, %d, %d\n", surface_area.x, surface_area.y, surface_area.w, surface_area.h);
	    fprintf(stderr, "\n");
#endif
	  }
	  remove_area = unique_area;
	}
      }
      // remove_area ŔwʂƂāAĕ`̈ɓo^
      for (std::vector<SDL_Rect>::iterator it = remove_area.begin();
	   it != remove_area.end(); ++it) {
	pimpl->back_rects.push_back(*it);
      }
    }
  }
}
