/*!
  \file
  \brief T[tFX

  \author Satofumi KAMIMURA

  $Id$
*/

#include "TextSurface.h"
#include "Surface.h"
#include "TtfResource.h"
#include "SdlUtils.h"
#include "SdlSurface.h"

using namespace beego;


struct TextSurface::pImpl {
  boost::shared_ptr<SdlSurface> surface;

  void initialize(TtfResource* font, const Uint16* text_data, size_t size,
		  Uint32 fore, Uint32 back, bool transparent) {

    // T[tFX̍쐬
    SDL_Color fore_color;
    set_SdlColor(&fore_color, fore);
    SDL_Color back_color;
    set_SdlColor(&back_color, back);
    boost::shared_ptr<SdlSurface>
      new_surface(new SdlSurface
		  (TTF_RenderUNICODE_Shaded(font->getResource(size),
					    &text_data[0],
					    fore_color, back_color),
		   transparent));
    std::swap(surface, new_surface);
  }

  pImpl(TtfResource* font, const char* text, size_t size,
	Uint32 fore, Uint32 back, bool transparent) {
    std::vector<Uint16> text_data;
    for (int i = 0; text[i] != 0x0; ++i) {
      text_data.push_back(text[i]);
    }
    text_data.push_back(0x0);
    initialize(font, &text_data[0], size, fore, back, transparent);
  }

  pImpl(TtfResource* font, const Uint16* text, size_t size,
	Uint32 fore, Uint32 back, bool transparent) {
    initialize(font, text, size, fore, back, transparent);
  }

  ~pImpl(void) {
    //surface.clear();
  }
};


TextSurface::TextSurface(TtfResource* font,
			 const char* text, size_t size,
			 Uint32 fore_color, Uint32 back_color,
			 bool transparent)
  : pimpl(new pImpl(font, text, size, fore_color, back_color, transparent)) {
}


TextSurface::TextSurface(TtfResource* font,
			 const Uint16* text, size_t size,
			 Uint32 fore_color, Uint32 back_color,
			 bool transparent)
  : pimpl(new pImpl(font, text, size, fore_color, back_color, transparent)) {
}


TextSurface::~TextSurface(void) {
}


void TextSurface::draw(std::vector<SDL_Rect>& update_rects,
		       const SDL_Rect* pos, const SDL_Rect* area,
		       size_t ticks) {
  pimpl->surface->draw(update_rects, pos, area, ticks);
}


size_t TextSurface::getWidth(void) {
  return pimpl->surface->getWidth();
}


size_t TextSurface::getHeight(void) {
  return pimpl->surface->getHeight();
}


bool TextSurface::isChanged(void) {
  return pimpl->surface->isChanged();
}


bool TextSurface::isTransparent(void) {
  return pimpl->surface->isTransparent();
}


void TextSurface::setAlpha(size_t percent) {

  // lXV
  pimpl->surface->setAlpha(percent);
}
